fix doxygen comment of the init functions of all audio format handlers.
[paraslash.git] / fd.c
diff --git a/fd.c b/fd.c
index 4928850e38d7d01688702d8acb8e898da53e284d..df7921b4f393c4b4c2bcfc963c757c0a67aec661 100644 (file)
--- a/fd.c
+++ b/fd.c
@@ -20,6 +20,7 @@
 
 #include "para.h"
 
+#include <fcntl.h>
 #include <sys/select.h>
 
 #include "error.h"
@@ -46,8 +47,8 @@ int file_exists(const char *fn)
  * \param n the highest-numbered descriptor in any of the two sets, plus 1
  * \param readfds fds that should be checked for readability
  * \param writefds fds that should be checked for writablility
* \param timeout upper bound on the amount of time elapsed before select()
*  returns
  *
  * \return The return value of the underlying select() call.
  *
@@ -55,15 +56,16 @@ int file_exists(const char *fn)
  * \sa select(2) select_tut(2)
  */
 int para_select(int n, fd_set *readfds, fd_set *writefds,
-               struct timeval *timeout)
+               struct timeval *timeout_tv)
 {
        int ret, err;
        do {
-               ret = select(n, readfds, writefds, NULL, timeout);
+               ret = select(n, readfds, writefds, NULL, timeout_tv);
                err = errno;
        } while (ret < 0 && err == EINTR);
        if (ret < 0)
-               PARA_CRIT_LOG("select error (%s)\n", strerror(err));
+               PARA_CRIT_LOG("select error: %s, max_fileno: %d\n",
+                       strerror(err), n);
        return ret;
 }
 
@@ -93,9 +95,80 @@ int mark_fd_nonblock(int fd)
  *
  * This wrapper for FD_SET() passes its first two arguments to \p FD_SET. Upon
  * return, \a max_fileno contains the maximum of the old_value and \a fd.
+ *
+ * \sa para_select
 */
 void para_fd_set(int fd, fd_set *fds, int *max_fileno)
 {
+
+       if (fd < 0 || fd >= FD_SETSIZE) {
+               PARA_EMERG_LOG("fatal: tried to add invalid fd %d\n", fd);
+               exit(EXIT_FAILURE);
+       }
+#if 0
+       {
+               int flags = fcntl(fd, F_GETFL);
+               if (!(flags & O_NONBLOCK)) {
+                       PARA_EMERG_LOG("fd %d is a blocking file descriptor\n", fd);
+                       exit(EXIT_FAILURE);
+               }
+       }
+#endif
        FD_SET(fd, fds);
-       *max_fileno = MAX(*max_fileno, fd);
+       *max_fileno = PARA_MAX(*max_fileno, fd);
+}
+
+/**
+ * paraslash's wrapper for fread(3)
+ *
+ * \param dest destination pointer
+ * \param nbytes size of one element
+ * \param nmemb number of elements to write
+ * \param stream the input stream
+ *
+ * \return negative on errors, zero on end of file, and the number of bytes
+ * (not elements) read on success.
+ *
+ * \sa fread(3)
+ */
+__must_check int para_fread(void *dest, size_t nbytes, size_t nmemb, FILE *stream)
+{
+       size_t res = fread(dest, nbytes, nmemb, stream);
+       if (res == nmemb)
+               return nbytes * nmemb;
+       if (feof(stream))
+               return 0;
+       return -E_FREAD;
+}
+/**
+* paraslash's wrapper for fgets(3)
+* \param line pointer to the buffer to store the line
+* \param size the size of the buffer given by \a line
+* \param f the stream to read from
+*
+* \return Unlike the standard fgets() function, an integer value
+* is returned. On success, this function returns 1. On errors, -E_FGETS
+* is returned. A zero return value indicates an end of file condition.
+*/
+__must_check int para_fgets(char *line, int size, FILE *f)
+{
+again:
+       if (fgets(line, size, f))
+               return 1;
+       if (feof(f))
+               return 0;
+       if (!ferror(f))
+               return -E_FGETS;
+       if (errno != EINTR) {
+               PARA_ERROR_LOG("%s\n", strerror(errno));
+               return -E_FGETS;
+       }
+       clearerr(f);
+       goto again;
+}
+
+int para_fseek(FILE *stream, long offset, int whence)
+{
+       int ret = fseek(stream, offset, whence);
+       return ret < 0? -E_FSEEK : ret;
 }
@@ -46,8 +47,8 @@ int file_exists(const char *fn)
  * \param readfds fds that should be checked for readability
  * \param writefds fds that should be checked for writablility
- * \param timeout upper bound on the amount of time elapsed before select()
- *  returns
+ * \param timeout_tv upper bound on the amount of time elapsed before select()
+ * returns
  *
  * \return The return value of the underlying select() call.